#
# HTMLTidy
#
# Wrapper class for the HTML-Tidy program
#
# Copyright (c) 2002 by Philipp "philiKON" von Weitershausen
#                       philikon@philikon.de
#

"""
HTMLTidy
"""

import popen2, re
from types import StringType

class HTMLTidy:
    """
    Python interface to the 'tidy' program.
    This class doesn't do anything upon instanciation, so you may create as
    many as you like.
    """

    def checkTidy(self):
        """
        Check whether the tidy program is callable
        """
        stdout, stderr = self.callCmd("tidy -help")
        if len(stdout):
            return 1
        return 0

    def tidyErrors(self, html, **kw):
        """
        Analyze HTML source 'html' and return a sequence of dictionaries
        containing the error information
        """
        valid_kw = ("ascii", "latin1", "iso2022", "utf8", "mac", "xml")
        cmd = "tidy -quiet -errors"
        
        for key, value in kw.items():
            if key not in valid_kw:
                raise Exception("Invalid keyword argument: %s=%s" % (key, value))
            if type(value) is not StringType:
                value = ""
            cmd += " -%s %s" % (key, value)
            
        stdout, stderr = self.callCmd(cmd, html)
        errors = self.parseErrors(stderr)

        # add the error lines to the dictionaries
        srclines = html.split("\n")
        for error in errors:
            error['code'] = srclines[error['line']-1]
        return errors

    def tidyUp(self, html, **kw):
        """
        Return a tidied up version of HTML source 'html'.
        Returns None if the document contains errors
        """
        valid_kw = ("indent", "omit", "wrap", "upper", "clean", "raw",
                    "ascii", "latin1", "iso2022", "utf8", "mac", "numeric",
                    "xml", "asxml", "slides")
        cmd = "tidy -quiet"
        
        for key, value in kw.items():
            if key not in valid_kw:
                raise Exception("Invalid keyword argument: %s=%s" % (key, value))
            if type(value) is not StringType:
                value = ""
            cmd += " -%s %s" % (key, value)
            
        stdout, stderr = self.callCmd(cmd, html)
        if stdout == "" and stderr.find("Error:") != -1:
            return None
        return stdout

    def callCmd(self, cmd, pipe=None):
        """
        Call command 'cmd'. Optionally, write pipe to its stdin.
        Return the program's output on stdout and stderr as strings.
        """
        stdout, stdin, stderr = popen2.popen3(cmd)
        if pipe:
            stdin.write(pipe)
            stdin.flush()
        stdin.close()
        return stdout.read(), stderr.read()

    def parseErrors(self, errstr):
        """
        Parse tidy's error output and return a sequence of dictionaries
        """
        expr = "line (?P<line>[0-9]+) column (?P<column>[0-9]+) - (?P<type>.*?): (?P<message>.*)"
        pattern = re.compile(expr)
        errors = []
        lines = errstr.split('\n')
        for line in lines:
            match = pattern.match(line)
            if match is not None:
                groupdict = match.groupdict()
                groupdict['line'] = int(groupdict['line'])
                groupdict['column'] = int(groupdict['column'])                
                errors.append(groupdict)
        return tuple(errors)

if __name__ == "__main__":
    print HTMLTidy().checkTidy()

